@@ -23,8 +23,8 @@
 
 #include "absl/flags/flag.h"
 #include "absl/flags/parse.h"
+#include "absl/log/check.h"
 
-#include <grpc/support/log.h>
 #include <grpcpp/grpcpp.h>
 
 #ifdef BAZEL_BUILD
@@ -88,7 +88,7 @@ class GreeterClient {
 
       // Verify that the request was completed successfully. Note that "ok"
       // corresponds solely to the request for updates introduced by Finish().
-      GPR_ASSERT(ok);
+      CHECK(ok);
 
       if (call->status.ok())
         std::cout << "Greeter received: " << call->reply.message() << std::endl;
