@@ -17,8 +17,6 @@
 #ifndef GRPC_SRC_CORE_EXT_TRANSPORT_CHTTP2_TRANSPORT_HTTP2_SETTINGS_H
 #define GRPC_SRC_CORE_EXT_TRANSPORT_CHTTP2_TRANSPORT_HTTP2_SETTINGS_H
 
-#include <grpc/support/port_platform.h>
-
 #include <stdint.h>
 
 #include <cstdint>
@@ -27,9 +25,11 @@
 #include "absl/strings/string_view.h"
 #include "absl/types/optional.h"
 
+#include <grpc/support/port_platform.h>
+
 #include "src/core/ext/transport/chttp2/transport/frame.h"
-#include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/transport/http2_errors.h"
+#include "src/core/util/useful.h"
 
 namespace grpc_core {
 
