@@ -16,10 +16,8 @@
 //
 //
 
-#ifndef GRPC_CORE_LIB_CHANNEL_STATUS_UTIL_H
-#define GRPC_CORE_LIB_CHANNEL_STATUS_UTIL_H
-
-#include <grpc/support/port_platform.h>
+#ifndef GRPC_SRC_CORE_LIB_CHANNEL_STATUS_UTIL_H
+#define GRPC_SRC_CORE_LIB_CHANNEL_STATUS_UTIL_H
 
 #include <string>
 
@@ -27,6 +25,7 @@
 #include "absl/strings/string_view.h"
 
 #include <grpc/status.h>
+#include <grpc/support/port_platform.h>
 
 /// If \a status_str is a valid status string, sets \a status to the
 /// corresponding status value and returns true.
@@ -78,4 +77,4 @@ absl::Status MaybeRewriteIllegalStatusCode(absl::Status status,
 
 }  // namespace grpc_core
 
-#endif  // GRPC_CORE_LIB_CHANNEL_STATUS_UTIL_H
+#endif  // GRPC_SRC_CORE_LIB_CHANNEL_STATUS_UTIL_H
