@@ -1,37 +1,41 @@
-/*
- *
- * Copyright 2015 gRPC authors.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- *
- */
-
-#include <grpc/support/port_platform.h>
+//
+//
+// Copyright 2015 gRPC authors.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
+//
 
 #include "src/core/lib/compression/compression_internal.h"
 
 #include <stdlib.h>
-#include <string.h>
+
+#include <string>
 
 #include "absl/container/inlined_vector.h"
+#include "absl/log/check.h"
 #include "absl/strings/ascii.h"
+#include "absl/strings/str_format.h"
 #include "absl/strings/str_split.h"
 
-#include <grpc/support/log.h>
+#include <grpc/compression.h>
+#include <grpc/support/port_platform.h>
 
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/debug/trace.h"
-#include "src/core/lib/surface/api_trace.h"
+#include "src/core/lib/gprpp/crash.h"
+#include "src/core/lib/gprpp/ref_counted_ptr.h"
+#include "src/core/lib/gprpp/ref_counted_string.h"
 
 namespace grpc_core {
 
@@ -107,24 +111,21 @@ absl::optional<grpc_compression_algorithm> ParseCompressionAlgorithm(
 grpc_compression_algorithm
 CompressionAlgorithmSet::CompressionAlgorithmForLevel(
     grpc_compression_level level) const {
-  GRPC_API_TRACE("grpc_message_compression_algorithm_for_level(level=%d)", 1,
-                 ((int)level));
   if (level > GRPC_COMPRESS_LEVEL_HIGH) {
-    gpr_log(GPR_ERROR, "Unknown message compression level %d.",
-            static_cast<int>(level));
-    abort();
+    Crash(absl::StrFormat("Unknown message compression level %d.",
+                          static_cast<int>(level)));
   }
 
   if (level == GRPC_COMPRESS_LEVEL_NONE) {
     return GRPC_COMPRESS_NONE;
   }
 
-  GPR_ASSERT(level > 0);
+  CHECK_GT(level, 0);
 
-  /* Establish a "ranking" or compression algorithms in increasing order of
-   * compression.
-   * This is simplistic and we will probably want to introduce other dimensions
-   * in the future (cpu/memory cost, etc). */
+  // Establish a "ranking" or compression algorithms in increasing order of
+  // compression.
+  // This is simplistic and we will probably want to introduce other dimensions
+  // in the future (cpu/memory cost, etc).
   absl::InlinedVector<grpc_compression_algorithm,
                       GRPC_COMPRESS_ALGORITHMS_COUNT>
       algos;
@@ -140,7 +141,7 @@ CompressionAlgorithmSet::CompressionAlgorithmForLevel(
 
   switch (level) {
     case GRPC_COMPRESS_LEVEL_NONE:
-      abort(); /* should have been handled already */
+      abort();  // should have been handled already
     case GRPC_COMPRESS_LEVEL_LOW:
       return algos[0];
     case GRPC_COMPRESS_LEVEL_MED:
@@ -163,19 +164,13 @@ CompressionAlgorithmSet CompressionAlgorithmSet::FromUint32(uint32_t value) {
 }
 
 CompressionAlgorithmSet CompressionAlgorithmSet::FromChannelArgs(
-    const grpc_channel_args* args) {
+    const ChannelArgs& args) {
   CompressionAlgorithmSet set;
   static const uint32_t kEverything =
       (1u << GRPC_COMPRESS_ALGORITHMS_COUNT) - 1;
-  if (args != nullptr) {
-    set = CompressionAlgorithmSet::FromUint32(grpc_channel_args_find_integer(
-        args, GRPC_COMPRESSION_CHANNEL_ENABLED_ALGORITHMS_BITSET,
-        grpc_integer_options{kEverything, 0, kEverything}));
-    set.Set(GRPC_COMPRESS_NONE);
-  } else {
-    set = CompressionAlgorithmSet::FromUint32(kEverything);
-  }
-  return set;
+  return CompressionAlgorithmSet::FromUint32(
+      args.GetInt(GRPC_COMPRESSION_CHANNEL_ENABLED_ALGORITHMS_BITSET)
+          .value_or(kEverything));
 }
 
 CompressionAlgorithmSet::CompressionAlgorithmSet() = default;
@@ -230,20 +225,50 @@ uint32_t CompressionAlgorithmSet::ToLegacyBitmask() const {
 }
 
 absl::optional<grpc_compression_algorithm>
-DefaultCompressionAlgorithmFromChannelArgs(const grpc_channel_args* args) {
-  if (args == nullptr) return absl::nullopt;
-  for (size_t i = 0; i < args->num_args; i++) {
-    if (strcmp(args->args[i].key, GRPC_COMPRESSION_CHANNEL_DEFAULT_ALGORITHM) ==
-        0) {
-      if (args->args[i].type == GRPC_ARG_INTEGER) {
-        return static_cast<grpc_compression_algorithm>(
-            args->args[i].value.integer);
-      } else if (args->args[i].type == GRPC_ARG_STRING) {
-        return ParseCompressionAlgorithm(args->args[i].value.string);
-      }
-    }
+DefaultCompressionAlgorithmFromChannelArgs(const ChannelArgs& args) {
+  auto* value = args.Get(GRPC_COMPRESSION_CHANNEL_DEFAULT_ALGORITHM);
+  if (value == nullptr) return absl::nullopt;
+  auto ival = value->GetIfInt();
+  if (ival.has_value()) {
+    return static_cast<grpc_compression_algorithm>(*ival);
+  }
+  auto sval = value->GetIfString();
+  if (sval != nullptr) {
+    return ParseCompressionAlgorithm(sval->as_string_view());
   }
   return absl::nullopt;
 }
 
+grpc_compression_options CompressionOptionsFromChannelArgs(
+    const ChannelArgs& args) {
+  // Set compression options.
+  grpc_compression_options compression_options;
+  grpc_compression_options_init(&compression_options);
+  auto default_level = args.GetInt(GRPC_COMPRESSION_CHANNEL_DEFAULT_LEVEL);
+  if (default_level.has_value()) {
+    compression_options.default_level.is_set = true;
+    compression_options.default_level.level = Clamp(
+        static_cast<grpc_compression_level>(*default_level),
+        GRPC_COMPRESS_LEVEL_NONE,
+        static_cast<grpc_compression_level>(GRPC_COMPRESS_LEVEL_COUNT - 1));
+  }
+  auto default_algorithm =
+      args.GetInt(GRPC_COMPRESSION_CHANNEL_DEFAULT_ALGORITHM);
+  if (default_algorithm.has_value()) {
+    compression_options.default_algorithm.is_set = true;
+    compression_options.default_algorithm.algorithm =
+        Clamp(static_cast<grpc_compression_algorithm>(*default_algorithm),
+              GRPC_COMPRESS_NONE,
+              static_cast<grpc_compression_algorithm>(
+                  GRPC_COMPRESS_ALGORITHMS_COUNT - 1));
+  }
+  auto enabled_algorithms_bitset =
+      args.GetInt(GRPC_COMPRESSION_CHANNEL_ENABLED_ALGORITHMS_BITSET);
+  if (enabled_algorithms_bitset.has_value()) {
+    compression_options.enabled_algorithms_bitset =
+        *enabled_algorithms_bitset | 1 /* always support no compression */;
+  }
+  return compression_options;
+}
+
 }  // namespace grpc_core
