@@ -16,14 +16,13 @@
 //
 //
 
-#ifndef GRPC_CORE_LIB_IOMGR_TIMER_H
-#define GRPC_CORE_LIB_IOMGR_TIMER_H
-
-#include <grpc/support/port_platform.h>
+#ifndef GRPC_SRC_CORE_LIB_IOMGR_TIMER_H
+#define GRPC_SRC_CORE_LIB_IOMGR_TIMER_H
 
 #include <cstdint>
 
 #include <grpc/event_engine/event_engine.h>
+#include <grpc/support/port_platform.h>
 #include <grpc/support/time.h>
 
 #include "src/core/lib/iomgr/exec_ctx.h"
@@ -133,4 +132,4 @@ void grpc_kick_poller(void);
 // Sets the timer implementation
 void grpc_set_timer_impl(grpc_timer_vtable* vtable);
 
-#endif  // GRPC_CORE_LIB_IOMGR_TIMER_H
+#endif  // GRPC_SRC_CORE_LIB_IOMGR_TIMER_H
