@@ -16,8 +16,6 @@
 //
 //
 
-#include <grpc/support/port_platform.h>
-
 #include "src/cpp/ext/filters/census/context.h"
 
 #include <new>
@@ -27,7 +25,10 @@
 #include "opencensus/trace/context_util.h"
 #include "opencensus/trace/propagation/grpc_trace_bin.h"
 
+#include <grpc/support/port_platform.h>
+
 #include "src/core/lib/transport/transport.h"
+#include "src/cpp/ext/filters/census/grpc_plugin.h"
 #include "src/cpp/ext/filters/census/rpc_encoding.h"
 
 namespace grpc {
@@ -41,13 +42,24 @@ void GenerateServerContext(absl::string_view tracing, absl::string_view method,
   // Destruct the current CensusContext to free the Span memory before
   // overwriting it below.
   context->~CensusContext();
+  if (method.empty()) {
+    new (context) CensusContext(grpc::internal::OpenCensusRegistry::Get()
+                                    .PopulateTagMapWithConstantLabels({}));
+    return;
+  }
   SpanContext parent_ctx =
       opencensus::trace::propagation::FromGrpcTraceBinHeader(tracing);
   if (parent_ctx.IsValid()) {
-    new (context) CensusContext(method, parent_ctx);
-    return;
+    new (context) CensusContext(method, parent_ctx,
+                                grpc::internal::OpenCensusRegistry::Get()
+                                    .PopulateTagMapWithConstantLabels({}));
+  } else {
+    new (context)
+        CensusContext(method, grpc::internal::OpenCensusRegistry::Get()
+                                  .PopulateTagMapWithConstantLabels({}));
   }
-  new (context) CensusContext(method, TagMap{});
+  grpc::internal::OpenCensusRegistry::Get()
+      .PopulateCensusContextWithConstantAttributes(context);
 }
 
 void GenerateClientContext(absl::string_view method, CensusContext* ctxt,
@@ -55,23 +67,36 @@ void GenerateClientContext(absl::string_view method, CensusContext* ctxt,
   // Destruct the current CensusContext to free the Span memory before
   // overwriting it below.
   ctxt->~CensusContext();
+  if (method.empty()) {
+    new (ctxt) CensusContext(grpc::internal::OpenCensusRegistry::Get()
+                                 .PopulateTagMapWithConstantLabels({}));
+    return;
+  }
   if (parent_ctxt != nullptr) {
     SpanContext span_ctxt = parent_ctxt->Context();
     Span span = parent_ctxt->Span();
     if (span_ctxt.IsValid()) {
-      new (ctxt) CensusContext(method, &span, TagMap{});
+      new (ctxt) CensusContext(method, &span,
+                               grpc::internal::OpenCensusRegistry::Get()
+                                   .PopulateTagMapWithConstantLabels({}));
+      grpc::internal::OpenCensusRegistry::Get()
+          .PopulateCensusContextWithConstantAttributes(ctxt);
       return;
     }
   }
   const Span& span = opencensus::trace::GetCurrentSpan();
-  const TagMap& tags = opencensus::tags::GetCurrentTagMap();
+  const TagMap& tags = grpc::internal::OpenCensusRegistry::Get()
+                           .PopulateTagMapWithConstantLabels(
+                               opencensus::tags::GetCurrentTagMap());
   if (span.context().IsValid()) {
     // Create span with parent.
     new (ctxt) CensusContext(method, &span, tags);
-    return;
+  } else {
+    // Create span without parent.
+    new (ctxt) CensusContext(method, tags);
   }
-  // Create span without parent.
-  new (ctxt) CensusContext(method, tags);
+  grpc::internal::OpenCensusRegistry::Get()
+      .PopulateCensusContextWithConstantAttributes(ctxt);
 }
 
 size_t TraceContextSerialize(const ::opencensus::trace::SpanContext& context,
@@ -92,13 +117,14 @@ size_t StatsContextSerialize(size_t /*max_tags_len*/, grpc_slice* /*tags*/) {
 
 size_t ServerStatsSerialize(uint64_t server_elapsed_time, char* buf,
                             size_t buf_size) {
-  return RpcServerStatsEncoding::Encode(server_elapsed_time, buf, buf_size);
+  return internal::RpcServerStatsEncoding::Encode(server_elapsed_time, buf,
+                                                  buf_size);
 }
 
 size_t ServerStatsDeserialize(const char* buf, size_t buf_size,
                               uint64_t* server_elapsed_time) {
-  return RpcServerStatsEncoding::Decode(absl::string_view(buf, buf_size),
-                                        server_elapsed_time);
+  return internal::RpcServerStatsEncoding::Decode(
+      absl::string_view(buf, buf_size), server_elapsed_time);
 }
 
 uint64_t GetIncomingDataSize(const grpc_call_final_info* final_info) {
