@@ -18,6 +18,7 @@
 
 #include <memory>
 
+#include "absl/strings/str_cat.h"
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
 
@@ -28,6 +29,7 @@
 #include "src/core/lib/gprpp/time.h"
 #include "test/core/end2end/end2end_tests.h"
 #include "test/core/end2end/tests/cancel_test_helpers.h"
+#include "test/core/test_util/test_config.h"
 
 namespace grpc_core {
 namespace {
@@ -36,8 +38,8 @@ void CancelAfterRoundTrip(CoreEnd2endTest& test,
                           std::unique_ptr<CancellationMode> mode,
                           Duration timeout) {
   auto c = test.NewClientCall("/service/method").Timeout(timeout).Create();
-  CoreEnd2endTest::IncomingMetadata server_initial_metadata;
-  CoreEnd2endTest::IncomingMessage server_message;
+  IncomingMetadata server_initial_metadata;
+  IncomingMessage server_message;
   c.NewBatch(1)
       .SendInitialMetadata({})
       .SendMessage(RandomSlice(100))
@@ -46,7 +48,7 @@ void CancelAfterRoundTrip(CoreEnd2endTest& test,
   auto s = test.RequestCall(101);
   test.Expect(101, true);
   test.Step();
-  CoreEnd2endTest::IncomingMessage client_message;
+  IncomingMessage client_message;
   s.NewBatch(102)
       .RecvMessage(client_message)
       .SendInitialMetadata({})
@@ -54,11 +56,11 @@ void CancelAfterRoundTrip(CoreEnd2endTest& test,
   test.Expect(102, true);
   test.Expect(1, true);
   test.Step();
-  CoreEnd2endTest::IncomingMessage server_message_2;
-  CoreEnd2endTest::IncomingStatusOnClient server_status;
+  IncomingMessage server_message_2;
+  IncomingStatusOnClient server_status;
   c.NewBatch(2).RecvMessage(server_message_2).RecvStatusOnClient(server_status);
   mode->Apply(c);
-  CoreEnd2endTest::IncomingCloseOnServer client_close;
+  IncomingCloseOnServer client_close;
   s.NewBatch(103).RecvCloseOnServer(client_close).SendMessage(RandomSlice(100));
   test.Expect(2, true);
   test.Expect(103, true);
@@ -83,14 +85,17 @@ CORE_END2END_TEST(CoreClientChannelTest,
   InitServer(ChannelArgs());
   InitClient(ChannelArgs().Set(
       GRPC_ARG_SERVICE_CONFIG,
-      "{\n"
-      "  \"methodConfig\": [ {\n"
-      "    \"name\": [\n"
-      "      { \"service\": \"service\", \"method\": \"method\" }\n"
-      "    ],\n"
-      "    \"timeout\": \"5s\"\n"
-      "  } ]\n"
-      "}"));
+      absl::StrCat(
+          "{\n"
+          "  \"methodConfig\": [ {\n"
+          "    \"name\": [\n"
+          "      { \"service\": \"service\", \"method\": \"method\" }\n"
+          "    ],\n"
+          "    \"timeout\": \"",
+          5 * grpc_test_slowdown_factor(),
+          "s\"\n"
+          "  } ]\n"
+          "}")));
   CancelAfterRoundTrip(*this, std::make_unique<DeadlineCancellationMode>(),
                        Duration::Infinity());
 }
