@@ -27,9 +27,6 @@ using testing::StrictMock;
 namespace grpc_core {
 
 namespace {
-// Offset a void pointer by a given amount
-void* Offset(void* base, size_t amt) { return static_cast<char*>(base) + amt; }
-
 // A mock activity that can be activated and deactivated.
 class MockActivity : public Activity, public Wakeable {
  public:
@@ -59,6 +56,11 @@ class MockActivity : public Activity, public Wakeable {
   std::unique_ptr<ScopedActivity> scoped_activity_;
 };
 
+#define EXPECT_WAKEUP(activity, statement)                                 \
+  EXPECT_CALL((activity), WakeupRequested()).Times(::testing::AtLeast(1)); \
+  statement;                                                               \
+  Mock::VerifyAndClearExpectations(&(activity));
+
 }  // namespace
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -67,17 +69,16 @@ class MockActivity : public Activity, public Wakeable {
 namespace filters_detail {
 
 TEST(LayoutTest, Empty) {
-  Layout<FallibleOperator<ClientMetadataHandle>> l;
+  Layout<ClientMetadataHandle> l;
   ASSERT_EQ(l.ops.size(), 0u);
   EXPECT_EQ(l.promise_size, 0u);
   EXPECT_EQ(l.promise_alignment, 0u);
 }
 
 TEST(LayoutTest, Add) {
-  Layout<FallibleOperator<ClientMetadataHandle>> l;
+  Layout<ClientMetadataHandle> l;
   l.Add(1, 4,
-        FallibleOperator<ClientMetadataHandle>{&l, 120, nullptr, nullptr,
-                                               nullptr});
+        Operator<ClientMetadataHandle>{&l, 120, nullptr, nullptr, nullptr});
   ASSERT_EQ(l.ops.size(), 1u);
   EXPECT_EQ(l.promise_size, 1u);
   EXPECT_EQ(l.promise_alignment, 4u);
@@ -328,7 +329,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningVoid) {
   EXPECT_EQ(d.client_initial_metadata.ops[0].poll, nullptr);
   EXPECT_EQ(d.client_initial_metadata.ops[0].early_destroy, nullptr);
   // Check promise init
-  auto md = Arena::MakePooled<ClientMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   char call_data;
   auto r = d.client_initial_metadata.ops[0].promise_init(
@@ -363,7 +364,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningVoidTakingChannelPtr) {
   EXPECT_EQ(d.client_initial_metadata.ops[0].poll, nullptr);
   EXPECT_EQ(d.client_initial_metadata.ops[0].early_destroy, nullptr);
   // Check promise init
-  auto md = Arena::MakePooled<ClientMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   char call_data;
   auto r = d.client_initial_metadata.ops[0].promise_init(
@@ -408,7 +409,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningAbslStatus) {
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
   auto arena = SimpleArenaAllocator()->MakeArena();
-  auto md = Arena::MakePooled<ClientMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -419,7 +420,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningAbslStatus) {
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>();
+  md = Arena::MakePooledForOverwrite<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       nullptr, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -466,7 +467,7 @@ TEST(StackDataTest,
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
   auto arena = SimpleArenaAllocator()->MakeArena();
-  auto md = Arena::MakePooled<ClientMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -477,7 +478,7 @@ TEST(StackDataTest,
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>();
+  md = Arena::MakePooledForOverwrite<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       nullptr, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -497,7 +498,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningServerMetadata) {
         md.Set(HttpPathMetadata(), Slice::FromStaticString("hello"));
         bool first = std::exchange(first_, false);
         return first ? nullptr
-                     : ServerMetadataFromStatus(absl::CancelledError());
+                     : ServerMetadataFromStatus(GRPC_STATUS_CANCELLED);
       }
 
      private:
@@ -523,7 +524,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningServerMetadata) {
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
   auto arena = SimpleArenaAllocator()->MakeArena();
-  auto md = Arena::MakePooled<ClientMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -534,7 +535,7 @@ TEST(StackDataTest, InstantClientInitialMetadataReturningServerMetadata) {
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>();
+  md = Arena::MakePooledForOverwrite<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       nullptr, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -556,7 +557,7 @@ TEST(StackDataTest,
         const bool first = std::exchange(first_, false);
         p->v.push_back(first ? 11 : 22);
         return first ? nullptr
-                     : ServerMetadataFromStatus(absl::CancelledError());
+                     : ServerMetadataFromStatus(GRPC_STATUS_CANCELLED);
       }
 
      private:
@@ -583,7 +584,7 @@ TEST(StackDataTest,
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
   auto arena = SimpleArenaAllocator()->MakeArena();
-  auto md = Arena::MakePooled<ClientMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -594,7 +595,7 @@ TEST(StackDataTest,
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>();
+  md = Arena::MakePooledForOverwrite<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       nullptr, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -641,7 +642,7 @@ TEST(StackDataTest, PromiseClientInitialMetadataReturningAbslStatus) {
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
   auto arena = SimpleArenaAllocator()->MakeArena();
-  auto md = Arena::MakePooled<ClientMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -659,7 +660,7 @@ TEST(StackDataTest, PromiseClientInitialMetadataReturningAbslStatus) {
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>();
+  md = Arena::MakePooledForOverwrite<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       promise_data, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -672,7 +673,7 @@ TEST(StackDataTest, PromiseClientInitialMetadataReturningAbslStatus) {
   EXPECT_EQ(r.value().ok, nullptr);
   EXPECT_EQ(r.value().error->get(GrpcStatusMetadata()), GRPC_STATUS_CANCELLED);
   // A cancelled call
-  md = Arena::MakePooled<ClientMetadata>();
+  md = Arena::MakePooledForOverwrite<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       promise_data, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -722,7 +723,7 @@ TEST(StackDataTest,
   void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
   d.filter_constructor[0].call_init(call_data, &f1);
   auto arena = SimpleArenaAllocator()->MakeArena();
-  auto md = Arena::MakePooled<ClientMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ClientMetadata>();
   promise_detail::Context<Arena> ctx(arena.get());
   // A succeeding call
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
@@ -740,7 +741,7 @@ TEST(StackDataTest,
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "hello");
   // A failing call
-  md = Arena::MakePooled<ClientMetadata>();
+  md = Arena::MakePooledForOverwrite<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       promise_data, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -753,7 +754,7 @@ TEST(StackDataTest,
   EXPECT_EQ(r.value().ok, nullptr);
   EXPECT_EQ(r.value().error->get(GrpcStatusMetadata()), GRPC_STATUS_CANCELLED);
   // A cancelled call
-  md = Arena::MakePooled<ClientMetadata>();
+  md = Arena::MakePooledForOverwrite<ClientMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = d.client_initial_metadata.ops[0].promise_init(
       promise_data, call_data, d.client_initial_metadata.ops[0].channel_data,
@@ -790,7 +791,7 @@ TEST(StackDataTest, InstantServerInitialMetadataReturningVoid) {
   EXPECT_EQ(d.server_initial_metadata.ops[0].early_destroy, nullptr);
   // Check promise init
   auto arena = SimpleArenaAllocator()->MakeArena();
-  auto md = Arena::MakePooled<ServerMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   char call_data;
   auto r = d.server_initial_metadata.ops[0].promise_init(
@@ -867,7 +868,7 @@ TEST(StackDataTest, InstantServerToClientMessagesReturningVoid) {
   EXPECT_EQ(r.value().ok->flags(), 1u);
 }
 
-TEST(StackDataTest, InstantServerTrailingMetadataReturningVoid) {
+TEST(StackDataTest, ServerTrailingMetadataReturningVoid) {
   struct Filter1 {
     struct Call {
       void OnServerTrailingMetadata(ServerMetadata& md) {
@@ -883,27 +884,20 @@ TEST(StackDataTest, InstantServerTrailingMetadataReturningVoid) {
   d.AddServerTrailingMetadataOp(&f1, call_offset);
   ASSERT_EQ(d.filter_constructor.size(), 0u);
   ASSERT_EQ(d.filter_destructor.size(), 0u);
-  ASSERT_EQ(d.server_trailing_metadata.ops.size(), 1u);
-  EXPECT_EQ(d.server_trailing_metadata.ops[0].call_offset, call_offset);
-  EXPECT_EQ(d.server_trailing_metadata.ops[0].channel_data, &f1);
-  // Instant => no poll/early destroy
-  EXPECT_EQ(d.server_trailing_metadata.ops[0].poll, nullptr);
-  EXPECT_EQ(d.server_trailing_metadata.ops[0].early_destroy, nullptr);
-  // Check promise init
+  ASSERT_EQ(d.server_trailing_metadata.size(), 1u);
+  EXPECT_EQ(d.server_trailing_metadata[0].call_offset, call_offset);
+  EXPECT_EQ(d.server_trailing_metadata[0].channel_data, &f1);
+  // Check operation
   auto arena = SimpleArenaAllocator()->MakeArena();
-  auto md = Arena::MakePooled<ServerMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   char call_data;
-  auto r = d.server_trailing_metadata.ops[0].promise_init(
-      nullptr, &call_data, d.server_trailing_metadata.ops[0].channel_data,
-      std::move(md));
-  EXPECT_TRUE(r.ready());
-  EXPECT_EQ(r.value()->get_pointer(HttpPathMetadata())->as_string_view(),
-            "hello");
+  auto r = d.server_trailing_metadata[0].server_trailing_metadata(
+      &call_data, d.server_trailing_metadata[0].channel_data, std::move(md));
+  EXPECT_EQ(r->get_pointer(HttpPathMetadata())->as_string_view(), "hello");
 }
 
-TEST(StackDataTest,
-     InstantServerTrailingMetadataReturningVoidTakingChannelPtr) {
+TEST(StackDataTest, ServerTrailingMetadataReturningVoidTakingChannelPtr) {
   struct Filter1 {
     struct Call {
       void OnServerTrailingMetadata(ServerMetadata& md, Filter1* p) {
@@ -921,23 +915,17 @@ TEST(StackDataTest,
   d.AddServerTrailingMetadataOp(&f1, call_offset);
   ASSERT_EQ(d.filter_constructor.size(), 0u);
   ASSERT_EQ(d.filter_destructor.size(), 0u);
-  ASSERT_EQ(d.server_trailing_metadata.ops.size(), 1u);
-  EXPECT_EQ(d.server_trailing_metadata.ops[0].call_offset, call_offset);
-  EXPECT_EQ(d.server_trailing_metadata.ops[0].channel_data, &f1);
-  // Instant => no poll/early destroy
-  EXPECT_EQ(d.server_trailing_metadata.ops[0].poll, nullptr);
-  EXPECT_EQ(d.server_trailing_metadata.ops[0].early_destroy, nullptr);
-  // Check promise init
+  ASSERT_EQ(d.server_trailing_metadata.size(), 1u);
+  EXPECT_EQ(d.server_trailing_metadata[0].call_offset, call_offset);
+  EXPECT_EQ(d.server_trailing_metadata[0].channel_data, &f1);
+  // Check operation
   auto arena = SimpleArenaAllocator()->MakeArena();
-  auto md = Arena::MakePooled<ServerMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   char call_data;
-  auto r = d.server_trailing_metadata.ops[0].promise_init(
-      nullptr, &call_data, d.server_trailing_metadata.ops[0].channel_data,
-      std::move(md));
-  EXPECT_TRUE(r.ready());
-  EXPECT_EQ(r.value()->get_pointer(HttpPathMetadata())->as_string_view(),
-            "hello");
+  auto r = d.server_trailing_metadata[0].server_trailing_metadata(
+      &call_data, d.server_trailing_metadata[0].channel_data, std::move(md));
+  EXPECT_EQ(r->get_pointer(HttpPathMetadata())->as_string_view(), "hello");
   EXPECT_THAT(f1.v, ::testing::ElementsAre(42));
 }
 
@@ -959,13 +947,13 @@ TEST(StackBuilderTest, AddOnServerTrailingMetadata) {
       [x = std::make_unique<int>(42)](ServerMetadata&) { EXPECT_EQ(*x, 42); });
   auto stack = b.Build();
   const auto& data = CallFilters::StackTestSpouse().StackDataFrom(*stack);
-  ASSERT_EQ(data.server_trailing_metadata.ops.size(), 1u);
+  ASSERT_EQ(data.server_trailing_metadata.size(), 1u);
   ASSERT_EQ(data.client_initial_metadata.ops.size(), 0u);
   ASSERT_EQ(data.client_to_server_messages.ops.size(), 0u);
   ASSERT_EQ(data.server_to_client_messages.ops.size(), 0u);
   ASSERT_EQ(data.server_initial_metadata.ops.size(), 0u);
-  EXPECT_EQ(data.server_trailing_metadata.ops[0].call_offset, 0);
-  EXPECT_NE(data.server_trailing_metadata.ops[0].channel_data, nullptr);
+  EXPECT_EQ(data.server_trailing_metadata[0].call_offset, 0);
+  EXPECT_NE(data.server_trailing_metadata[0].channel_data, nullptr);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1019,7 +1007,7 @@ TEST(OperationExecutorTest, InstantTwo) {
   auto arena = SimpleArenaAllocator()->MakeArena();
   promise_detail::Context<Arena> ctx(arena.get());
   // First call succeeds
-  auto md = Arena::MakePooled<ServerMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   auto r =
       transformer.Start(&d.client_initial_metadata, std::move(md), call_data1);
@@ -1027,7 +1015,7 @@ TEST(OperationExecutorTest, InstantTwo) {
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "world");
   // Second fails
-  md = Arena::MakePooled<ServerMetadata>();
+  md = Arena::MakePooledForOverwrite<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = transformer.Start(&d.client_initial_metadata, std::move(md), call_data1);
   EXPECT_TRUE(r.ready());
@@ -1082,7 +1070,7 @@ TEST(OperationExecutorTest, PromiseTwo) {
   auto arena = SimpleArenaAllocator()->MakeArena();
   promise_detail::Context<Arena> ctx(arena.get());
   // First call succeeds after two sets of two step delays.
-  auto md = Arena::MakePooled<ServerMetadata>();
+  auto md = Arena::MakePooledForOverwrite<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   auto r =
       transformer.Start(&d.client_initial_metadata, std::move(md), call_data1);
@@ -1098,7 +1086,7 @@ TEST(OperationExecutorTest, PromiseTwo) {
   EXPECT_EQ(r.value().ok->get_pointer(HttpPathMetadata())->as_string_view(),
             "world");
   // Second fails after one set of two step delays.
-  md = Arena::MakePooled<ServerMetadata>();
+  md = Arena::MakePooledForOverwrite<ServerMetadata>();
   EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
   r = transformer.Start(&d.client_initial_metadata, std::move(md), call_data1);
   EXPECT_FALSE(r.ready());
@@ -1113,187 +1101,6 @@ TEST(OperationExecutorTest, PromiseTwo) {
 
 }  // namespace filters_detail
 
-///////////////////////////////////////////////////////////////////////////////
-// InfallibleOperationExecutor
-
-namespace filters_detail {
-
-TEST(InfallibleOperationExecutor, NoOp) {
-  OperationExecutor<ServerMetadataHandle> pipe;
-  EXPECT_FALSE(pipe.IsRunning());
-}
-
-TEST(InfallibleOperationExecutor, InstantTwo) {
-  class Filter1 {
-   public:
-    class Call {
-     public:
-      void OnServerTrailingMetadata(ClientMetadata& md) {
-        if (md.get_pointer(HttpPathMetadata()) != nullptr) {
-          md.Set(HttpPathMetadata(), Slice::FromStaticString("world"));
-        } else {
-          md.Set(HttpPathMetadata(), Slice::FromStaticString("hello"));
-        }
-      }
-    };
-  };
-  StackData d;
-  Filter1 f1;
-  Filter1 f2;
-  const size_t call_offset1 = d.AddFilter(&f1);
-  const size_t call_offset2 = d.AddFilter(&f2);
-  d.AddServerTrailingMetadataOp(&f1, call_offset1);
-  d.AddServerTrailingMetadataOp(&f2, call_offset2);
-  ASSERT_EQ(d.filter_constructor.size(), 0u);
-  ASSERT_EQ(d.filter_destructor.size(), 0u);
-  ASSERT_EQ(d.server_trailing_metadata.ops.size(), 2u);
-  void* call_data = gpr_malloc_aligned(d.call_data_size, d.call_data_alignment);
-  InfallibleOperationExecutor<ServerMetadataHandle> transformer;
-  auto arena = SimpleArenaAllocator()->MakeArena();
-  promise_detail::Context<Arena> ctx(arena.get());
-  auto md = Arena::MakePooled<ServerMetadata>();
-  EXPECT_EQ(md->get_pointer(HttpPathMetadata()), nullptr);
-  auto r =
-      transformer.Start(&d.server_trailing_metadata, std::move(md), call_data);
-  EXPECT_TRUE(r.ready());
-  EXPECT_EQ(r.value()->get_pointer(HttpPathMetadata())->as_string_view(),
-            "world");
-  gpr_free_aligned(call_data);
-}
-
-}  // namespace filters_detail
-
-///////////////////////////////////////////////////////////////////////////////
-// PipeState
-
-namespace filters_detail {
-
-TEST(PipeStateTest, NoOp) { PipeState(); }
-
-TEST(PipeStateTest, OnePull) {
-  PipeState ps;
-  StrictMock<MockActivity> activity;
-  activity.Activate();
-  // initially: not started, should only see pending from pulls
-  EXPECT_THAT(ps.PollPull(), IsPending());
-  EXPECT_THAT(ps.PollPull(), IsPending());
-  // start it, should see a wakeup
-  EXPECT_CALL(activity, WakeupRequested());
-  ps.Start();
-  Mock::VerifyAndClearExpectations(&activity);
-  // should still see pending! nothing's been pushed
-  EXPECT_THAT(ps.PollPull(), IsPending());
-  EXPECT_THAT(ps.PollPull(), IsPending());
-  // begin a push, should see a wakeup
-  EXPECT_CALL(activity, WakeupRequested());
-  ps.BeginPush();
-  Mock::VerifyAndClearExpectations(&activity);
-  // now we should see a value on the pull poll
-  EXPECT_THAT(ps.PollPull(), IsReady(true));
-  // push should be pending though!
-  EXPECT_THAT(ps.PollPush(), IsPending());
-  // ack the pull, should see a wakeup
-  EXPECT_CALL(activity, WakeupRequested());
-  ps.AckPull();
-  Mock::VerifyAndClearExpectations(&activity);
-  // now the push is complete
-  EXPECT_THAT(ps.PollPush(), IsReady(Success()));
-  ps.DropPush();
-  ps.DropPull();
-  EXPECT_FALSE(ps.holds_error());
-}
-
-TEST(PipeStateTest, StartThenPull) {
-  PipeState ps;
-  StrictMock<MockActivity> activity;
-  activity.Activate();
-  ps.Start();
-  // pull is pending! nothing's been pushed
-  EXPECT_THAT(ps.PollPull(), IsPending());
-  EXPECT_THAT(ps.PollPull(), IsPending());
-  // begin a push, should see a wakeup
-  EXPECT_CALL(activity, WakeupRequested());
-  ps.BeginPush();
-  Mock::VerifyAndClearExpectations(&activity);
-  // now we should see a value on the pull poll
-  EXPECT_THAT(ps.PollPull(), IsReady(true));
-  // push should be pending though!
-  EXPECT_THAT(ps.PollPush(), IsPending());
-  // ack the pull, should see a wakeup
-  EXPECT_CALL(activity, WakeupRequested());
-  ps.AckPull();
-  Mock::VerifyAndClearExpectations(&activity);
-  // now the push is complete
-  EXPECT_THAT(ps.PollPush(), IsReady(Success()));
-  ps.DropPush();
-  ps.DropPull();
-  EXPECT_FALSE(ps.holds_error());
-}
-
-TEST(PipeStateTest, PushFirst) {
-  PipeState ps;
-  StrictMock<MockActivity> activity;
-  activity.Activate();
-  // start immediately, and push immediately
-  ps.Start();
-  ps.BeginPush();
-  // push should be pending
-  EXPECT_THAT(ps.PollPush(), IsPending());
-  // pull should immediately see a value
-  EXPECT_THAT(ps.PollPull(), IsReady(true));
-  // push should still be pending though!
-  EXPECT_THAT(ps.PollPush(), IsPending());
-  // ack the pull, should see a wakeup
-  EXPECT_CALL(activity, WakeupRequested());
-  ps.AckPull();
-  Mock::VerifyAndClearExpectations(&activity);
-  // now the push is complete
-  EXPECT_THAT(ps.PollPush(), IsReady(Success()));
-  ps.DropPush();
-  ps.DropPull();
-  EXPECT_FALSE(ps.holds_error());
-}
-
-TEST(PipeStateTest, DropPushing) {
-  PipeState ps;
-  StrictMock<MockActivity> activity;
-  activity.Activate();
-  ps.BeginPush();
-  ps.DropPush();
-  EXPECT_TRUE(ps.holds_error());
-  EXPECT_THAT(ps.PollPull(), IsReady(Failure()));
-  ps.BeginPush();
-  EXPECT_THAT(ps.PollPush(), IsReady(Failure()));
-  ps.DropPush();
-}
-
-TEST(PipeStateTest, DropPulling) {
-  PipeState ps;
-  StrictMock<MockActivity> activity;
-  activity.Activate();
-  EXPECT_THAT(ps.PollPull(), IsPending());
-  ps.DropPull();
-  EXPECT_TRUE(ps.holds_error());
-  EXPECT_THAT(ps.PollPull(), IsReady(Failure()));
-  ps.DropPull();
-  EXPECT_THAT(ps.PollPush(), IsReady(Failure()));
-}
-
-TEST(PipeStateTest, DropProcessing) {
-  PipeState ps;
-  StrictMock<MockActivity> activity;
-  activity.Activate();
-  ps.Start();
-  ps.BeginPush();
-  EXPECT_THAT(ps.PollPull(), IsReady(true));
-  ps.DropPull();
-  EXPECT_TRUE(ps.holds_error());
-  EXPECT_THAT(ps.PollPull(), IsReady(Failure()));
-  EXPECT_THAT(ps.PollPush(), IsReady(Failure()));
-}
-
-}  // namespace filters_detail
-
 ///////////////////////////////////////////////////////////////////////////////
 // CallFilters
 
@@ -1354,8 +1161,9 @@ TEST(CallFiltersTest, UnaryCall) {
   builder.Add(&f1);
   builder.Add(&f2);
   auto arena = SimpleArenaAllocator()->MakeArena();
-  CallFilters filters(Arena::MakePooled<ClientMetadata>());
-  filters.SetStack(builder.Build());
+  CallFilters filters(Arena::MakePooledForOverwrite<ClientMetadata>());
+  filters.AddStack(builder.Build());
+  filters.Start();
   promise_detail::Context<Arena> ctx(arena.get());
   StrictMock<MockActivity> activity;
   activity.Activate();
@@ -1369,35 +1177,125 @@ TEST(CallFiltersTest, UnaryCall) {
   EXPECT_THAT(push_client_to_server_message(), IsPending());
   auto pull_client_to_server_message = filters.PullClientToServerMessage();
   // Pull client to server message, expect a wakeup
-  EXPECT_CALL(activity, WakeupRequested());
-  EXPECT_THAT(pull_client_to_server_message(), IsReady());
-  Mock::VerifyAndClearExpectations(&activity);
+  EXPECT_WAKEUP(activity,
+                EXPECT_THAT(pull_client_to_server_message(), IsReady()));
   // Push should be done
   EXPECT_THAT(push_client_to_server_message(), IsReady(Success{}));
   // Push server initial metadata
-  auto push_server_initial_metadata =
-      filters.PushServerInitialMetadata(Arena::MakePooled<ServerMetadata>());
-  EXPECT_THAT(push_server_initial_metadata(), IsPending());
+  filters.PushServerInitialMetadata(
+      Arena::MakePooledForOverwrite<ServerMetadata>());
   auto pull_server_initial_metadata = filters.PullServerInitialMetadata();
-  // Pull server initial metadata, expect a wakeup
-  EXPECT_CALL(activity, WakeupRequested());
+  // Pull server initial metadata
   EXPECT_THAT(pull_server_initial_metadata(), IsReady());
   Mock::VerifyAndClearExpectations(&activity);
-  // Push should be done
-  EXPECT_THAT(push_server_initial_metadata(), IsReady(Success{}));
   // Push server to client message
   auto push_server_to_client_message = filters.PushServerToClientMessage(
       Arena::MakePooled<Message>(SliceBuffer(), 0));
   EXPECT_THAT(push_server_to_client_message(), IsPending());
   auto pull_server_to_client_message = filters.PullServerToClientMessage();
   // Pull server to client message, expect a wakeup
-  EXPECT_CALL(activity, WakeupRequested());
-  EXPECT_THAT(pull_server_to_client_message(), IsReady());
-  Mock::VerifyAndClearExpectations(&activity);
+  EXPECT_WAKEUP(activity,
+                EXPECT_THAT(pull_server_to_client_message(), IsReady()));
   // Push should be done
   EXPECT_THAT(push_server_to_client_message(), IsReady(Success{}));
   // Push server trailing metadata
-  filters.PushServerTrailingMetadata(Arena::MakePooled<ServerMetadata>());
+  filters.PushServerTrailingMetadata(
+      Arena::MakePooledForOverwrite<ServerMetadata>());
+  // Pull server trailing metadata
+  auto pull_server_trailing_metadata = filters.PullServerTrailingMetadata();
+  // Should be done
+  EXPECT_THAT(pull_server_trailing_metadata(), IsReady());
+  filters.Finalize(nullptr);
+  EXPECT_THAT(steps,
+              ::testing::ElementsAre(
+                  "f1:OnClientInitialMetadata", "f2:OnClientInitialMetadata",
+                  "f1:OnClientToServerMessage", "f2:OnClientToServerMessage",
+                  "f2:OnServerInitialMetadata", "f1:OnServerInitialMetadata",
+                  "f2:OnServerToClientMessage", "f1:OnServerToClientMessage",
+                  "f2:OnServerTrailingMetadata", "f1:OnServerTrailingMetadata",
+                  "f1:OnFinalize", "f2:OnFinalize"));
+}
+
+TEST(CallFiltersTest, UnaryCallWithMultiStack) {
+  struct Filter {
+    struct Call {
+      void OnClientInitialMetadata(ClientMetadata&, Filter* f) {
+        f->steps.push_back(absl::StrCat(f->label, ":OnClientInitialMetadata"));
+      }
+      void OnServerInitialMetadata(ServerMetadata&, Filter* f) {
+        f->steps.push_back(absl::StrCat(f->label, ":OnServerInitialMetadata"));
+      }
+      void OnClientToServerMessage(Message&, Filter* f) {
+        f->steps.push_back(absl::StrCat(f->label, ":OnClientToServerMessage"));
+      }
+      void OnClientToServerHalfClose(Filter* f) {
+        f->steps.push_back(
+            absl::StrCat(f->label, ":OnClientToServerHalfClose"));
+      }
+      void OnServerToClientMessage(Message&, Filter* f) {
+        f->steps.push_back(absl::StrCat(f->label, ":OnServerToClientMessage"));
+      }
+      void OnServerTrailingMetadata(ServerMetadata&, Filter* f) {
+        f->steps.push_back(absl::StrCat(f->label, ":OnServerTrailingMetadata"));
+      }
+      void OnFinalize(const grpc_call_final_info*, Filter* f) {
+        f->steps.push_back(absl::StrCat(f->label, ":OnFinalize"));
+      }
+      std::unique_ptr<int> i = std::make_unique<int>(3);
+    };
+
+    const std::string label;
+    std::vector<std::string>& steps;
+  };
+  std::vector<std::string> steps;
+  Filter f1{"f1", steps};
+  Filter f2{"f2", steps};
+  CallFilters::StackBuilder builder1;
+  CallFilters::StackBuilder builder2;
+  builder1.Add(&f1);
+  builder2.Add(&f2);
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  CallFilters filters(Arena::MakePooledForOverwrite<ClientMetadata>());
+  filters.AddStack(builder1.Build());
+  filters.AddStack(builder2.Build());
+  filters.Start();
+  promise_detail::Context<Arena> ctx(arena.get());
+  StrictMock<MockActivity> activity;
+  activity.Activate();
+  // Pull client initial metadata
+  auto pull_client_initial_metadata = filters.PullClientInitialMetadata();
+  EXPECT_THAT(pull_client_initial_metadata(), IsReady());
+  Mock::VerifyAndClearExpectations(&activity);
+  // Push client to server message
+  auto push_client_to_server_message = filters.PushClientToServerMessage(
+      Arena::MakePooled<Message>(SliceBuffer(), 0));
+  EXPECT_THAT(push_client_to_server_message(), IsPending());
+  auto pull_client_to_server_message = filters.PullClientToServerMessage();
+  // Pull client to server message, expect a wakeup
+  EXPECT_WAKEUP(activity,
+                EXPECT_THAT(pull_client_to_server_message(), IsReady()));
+  // Push should be done
+  EXPECT_THAT(push_client_to_server_message(), IsReady(Success{}));
+  // Push server initial metadata
+  filters.PushServerInitialMetadata(
+      Arena::MakePooledForOverwrite<ServerMetadata>());
+  auto pull_server_initial_metadata = filters.PullServerInitialMetadata();
+  // Pull server initial metadata
+  EXPECT_THAT(pull_server_initial_metadata(), IsReady());
+  Mock::VerifyAndClearExpectations(&activity);
+  // Push server to client message
+  auto push_server_to_client_message = filters.PushServerToClientMessage(
+      Arena::MakePooled<Message>(SliceBuffer(), 0));
+  EXPECT_THAT(push_server_to_client_message(), IsPending());
+  auto pull_server_to_client_message = filters.PullServerToClientMessage();
+  // Pull server to client message, expect a wakeup
+  EXPECT_WAKEUP(activity,
+                EXPECT_THAT(pull_server_to_client_message(), IsReady()));
+  // Push should be done
+  EXPECT_THAT(push_server_to_client_message(), IsReady(Success{}));
+  // Push server trailing metadata
+  filters.PushServerTrailingMetadata(
+      Arena::MakePooledForOverwrite<ServerMetadata>());
   // Pull server trailing metadata
   auto pull_server_trailing_metadata = filters.PullServerTrailingMetadata();
   // Should be done
@@ -1417,5 +1315,6 @@ TEST(CallFiltersTest, UnaryCall) {
 
 int main(int argc, char** argv) {
   ::testing::InitGoogleTest(&argc, argv);
+  grpc_tracer_init();
   return RUN_ALL_TESTS();
 }
