@@ -22,12 +22,14 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <memory>
+
 #include <gtest/gtest.h>
 
+#include "absl/types/span.h"
+
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 
-#include "src/core/lib/gprpp/crash.h"
 #include "test/core/tsi/alts/crypt/gsec_test_util.h"
 
 static void alts_crypter_test_random_seal_unseal(alts_crypter* server_seal,
@@ -354,28 +356,48 @@ static void create_random_alts_seal_crypter(
   size_t key_length = rekey ? kAes128GcmRekeyKeyLength : kAes128GcmKeyLength;
   uint8_t* key;
   gsec_test_random_array(&key, key_length);
-  gsec_aes_gcm_aead_crypter_create(key, key_length, kAesGcmNonceLength,
-                                   kAesGcmTagLength, rekey, server_crypter_seal,
-                                   nullptr);
-  gsec_aes_gcm_aead_crypter_create(key, key_length, kAesGcmNonceLength,
-                                   kAesGcmTagLength, rekey,
-                                   server_crypter_unseal, nullptr);
-  gsec_aes_gcm_aead_crypter_create(key, key_length, kAesGcmNonceLength,
-                                   kAesGcmTagLength, rekey, client_crypter_seal,
-                                   nullptr);
-  gsec_aes_gcm_aead_crypter_create(key, key_length, kAesGcmNonceLength,
-                                   kAesGcmTagLength, rekey,
-                                   client_crypter_unseal, nullptr);
+  ASSERT_EQ(gsec_aes_gcm_aead_crypter_create(
+                std::make_unique<grpc_core::GsecKey>(
+                    absl::MakeConstSpan(key, key_length), rekey),
+                kAesGcmNonceLength, kAesGcmTagLength, server_crypter_seal,
+                /*error_details=*/nullptr),
+            GRPC_STATUS_OK);
+  ASSERT_EQ(gsec_aes_gcm_aead_crypter_create(
+                std::make_unique<grpc_core::GsecKey>(
+                    absl::MakeConstSpan(key, key_length), rekey),
+                kAesGcmNonceLength, kAesGcmTagLength, server_crypter_unseal,
+                /*error_details=*/nullptr),
+            GRPC_STATUS_OK);
+  ASSERT_EQ(gsec_aes_gcm_aead_crypter_create(
+                std::make_unique<grpc_core::GsecKey>(
+                    absl::MakeConstSpan(key, key_length), rekey),
+                kAesGcmNonceLength, kAesGcmTagLength, client_crypter_seal,
+                /*error_details=*/nullptr),
+            GRPC_STATUS_OK);
+  ASSERT_EQ(gsec_aes_gcm_aead_crypter_create(
+                std::make_unique<grpc_core::GsecKey>(
+                    absl::MakeConstSpan(key, key_length), rekey),
+                kAesGcmNonceLength, kAesGcmTagLength, client_crypter_unseal,
+                /*error_details=*/nullptr),
+            GRPC_STATUS_OK);
 
   size_t overflow_size = rekey ? 8 : 5;
-  alts_seal_crypter_create(*client_crypter_seal, /*is_client=*/true,
-                           overflow_size, client_seal, nullptr);
-  alts_unseal_crypter_create(*client_crypter_unseal, /*is_client=*/true,
-                             overflow_size, client_unseal, nullptr);
-  alts_seal_crypter_create(*server_crypter_seal, /*is_client=*/false,
-                           overflow_size, server_seal, nullptr);
-  alts_unseal_crypter_create(*server_crypter_unseal, /*is_client=*/false,
-                             overflow_size, server_unseal, nullptr);
+  ASSERT_EQ(alts_seal_crypter_create(*client_crypter_seal, /*is_client=*/true,
+                                     overflow_size, client_seal,
+                                     /*error_details=*/nullptr),
+            GRPC_STATUS_OK);
+  ASSERT_EQ(alts_unseal_crypter_create(
+                *client_crypter_unseal, /*is_client=*/true, overflow_size,
+                client_unseal, /*error_details=*/nullptr),
+            GRPC_STATUS_OK);
+  ASSERT_EQ(alts_seal_crypter_create(*server_crypter_seal, /*is_client=*/false,
+                                     overflow_size, server_seal,
+                                     /*error_details=*/nullptr),
+            GRPC_STATUS_OK);
+  ASSERT_EQ(alts_unseal_crypter_create(
+                *server_crypter_unseal, /*is_client=*/false, overflow_size,
+                server_unseal, /*error_details=*/nullptr),
+            GRPC_STATUS_OK);
   gpr_free(key);
 }
 
