@@ -22,24 +22,23 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
+#include "absl/log/check.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 
 #include <grpc/grpc.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 #include <grpc/support/string_util.h>
 
 #include "src/core/lib/gprpp/crash.h"
-#include "src/core/lib/iomgr/load_file.h"
 #include "src/core/lib/security/security_connector/security_connector.h"
 #include "src/core/tsi/ssl_transport_security.h"
 #include "src/core/tsi/transport_security.h"
 #include "src/core/tsi/transport_security_interface.h"
+#include "test/core/test_util/test_config.h"
+#include "test/core/test_util/tls_utils.h"
 #include "test/core/tsi/transport_security_test_lib.h"
-#include "test/core/util/test_config.h"
-#include "test/core/util/tls_utils.h"
 
 extern "C" {
 #include <openssl/crypto.h>
@@ -69,6 +68,12 @@ const char* kRevokedIntermediateCertPath =
 const char* kRootCrlPath = "test/core/tsi/test_creds/crl_data/crls/current.crl";
 const char* kIntermediateCrlPath =
     "test/core/tsi/test_creds/crl_data/crls/intermediate.crl";
+const char* kModifiedSignaturePath =
+    "test/core/tsi/test_creds/crl_data/bad_crls/invalid_signature.crl";
+const char* kModifiedContentPath =
+    "test/core/tsi/test_creds/crl_data/bad_crls/invalid_content.crl";
+const char* kEvilCrlPath =
+    "test/core/tsi/test_creds/crl_data/bad_crls/evil.crl";
 
 class CrlSslTransportSecurityTest
     : public testing::TestWithParam<tsi_tls_version> {
@@ -109,7 +114,7 @@ class CrlSslTransportSecurityTest
           gpr_malloc(sizeof(tsi_ssl_pem_key_cert_pair)));
       client_pem_key_cert_pairs_[0].private_key = client_key_.c_str();
       client_pem_key_cert_pairs_[0].cert_chain = client_cert_.c_str();
-      GPR_ASSERT(root_store_ != nullptr);
+      CHECK_NE(root_store_, nullptr);
     }
 
     void Run() {
@@ -128,7 +133,7 @@ class CrlSslTransportSecurityTest
 
    private:
     static void SetupHandshakers(tsi_test_fixture* fixture) {
-      GPR_ASSERT(fixture != nullptr);
+      CHECK_NE(fixture, nullptr);
       auto* self = reinterpret_cast<SslTsiTestFixture*>(fixture);
       self->SetupHandshakers();
     }
@@ -173,7 +178,7 @@ class CrlSslTransportSecurityTest
     }
 
     static void CheckHandshakerPeers(tsi_test_fixture* fixture) {
-      GPR_ASSERT(fixture != nullptr);
+      CHECK_NE(fixture, nullptr);
       auto* self = reinterpret_cast<SslTsiTestFixture*>(fixture);
       self->CheckHandshakerPeers();
     }
@@ -201,10 +206,11 @@ class CrlSslTransportSecurityTest
       // Handshake succeeds because the CRL that revokes the cert is not
       // present.
       bool expect_server_success = expect_server_success_;
+      bool expect_client_success = false;
 #if OPENSSL_VERSION_NUMBER >= 0x10100000
-      bool expect_client_success = GetParam() == tsi_tls_version::TSI_TLS1_2
-                                       ? expect_client_success_1_2_
-                                       : expect_client_success_1_3_;
+      expect_client_success = GetParam() == tsi_tls_version::TSI_TLS1_2
+                                  ? expect_client_success_1_2_
+                                  : expect_client_success_1_3_;
 #else
       //  If using OpenSSL version < 1.1, the CRL revocation won't
       //  be enabled anyways, so we always expect the connection to
@@ -414,10 +420,55 @@ TEST_P(CrlSslTransportSecurityTest,
 std::string TestNameSuffix(
     const ::testing::TestParamInfo<tsi_tls_version>& version) {
   if (version.param == tsi_tls_version::TSI_TLS1_2) return "TLS_1_2";
-  GPR_ASSERT(version.param == tsi_tls_version::TSI_TLS1_3);
+  CHECK(version.param == tsi_tls_version::TSI_TLS1_3);
   return "TLS_1_3";
 }
 
+TEST_P(CrlSslTransportSecurityTest, CrlProviderModifiedContentCrl) {
+  std::string root_crl =
+      grpc_core::testing::GetFileContents(kModifiedContentPath);
+  std::string intermediate_crl =
+      grpc_core::testing::GetFileContents(kIntermediateCrlPath);
+
+  absl::StatusOr<std::shared_ptr<grpc_core::experimental::CrlProvider>>
+      provider = grpc_core::experimental::CreateStaticCrlProvider(
+          {root_crl, intermediate_crl});
+  ASSERT_NE(provider.status(), absl::OkStatus()) << provider.status();
+}
+
+TEST_P(CrlSslTransportSecurityTest, CrlProviderModifiedSignatureCrl) {
+  std::string root_crl =
+      grpc_core::testing::GetFileContents(kModifiedSignaturePath);
+  std::string intermediate_crl =
+      grpc_core::testing::GetFileContents(kIntermediateCrlPath);
+
+  absl::StatusOr<std::shared_ptr<grpc_core::experimental::CrlProvider>>
+      provider = grpc_core::experimental::CreateStaticCrlProvider(
+          {root_crl, intermediate_crl});
+  ASSERT_TRUE(provider.ok()) << provider.status();
+
+  auto* fixture = new SslTsiTestFixture(kValidKeyPath, kValidCertPath,
+                                        kValidKeyPath, kValidCertPath, nullptr,
+                                        *provider, false, false, false);
+  fixture->Run();
+}
+
+TEST_P(CrlSslTransportSecurityTest, CrlFromBadCa) {
+  std::string root_crl = grpc_core::testing::GetFileContents(kEvilCrlPath);
+  std::string intermediate_crl =
+      grpc_core::testing::GetFileContents(kIntermediateCrlPath);
+
+  absl::StatusOr<std::shared_ptr<grpc_core::experimental::CrlProvider>>
+      provider = grpc_core::experimental::CreateStaticCrlProvider(
+          {root_crl, intermediate_crl});
+  ASSERT_TRUE(provider.ok()) << provider.status();
+
+  auto* fixture = new SslTsiTestFixture(kValidKeyPath, kValidCertPath,
+                                        kValidKeyPath, kValidCertPath, nullptr,
+                                        *provider, false, false, false);
+  fixture->Run();
+}
+
 // TODO(gtcooke94) Add nullptr issuer test cases - this is not simple to test
 // the way the code is currently designed - we plan to refactor ways the OpenSSL
 // callback functions are written to have more easily testable chunks in
