@@ -29,7 +29,7 @@ import sys
 import yaml
 
 with open("src/core/lib/config/config_vars.yaml") as f:
-    attrs = yaml.safe_load(f.read(), Loader=yaml.FullLoader)
+    attrs = yaml.safe_load(f.read())
 
 error = False
 today = datetime.date.today()
@@ -39,7 +39,9 @@ for attr in attrs:
         print("config has no name: %r" % attr)
         error = True
         continue
-    if "experiment" in attr["name"] and attr["name"] != "experiments":
+    if (
+        "experiment" in attr["name"] and "experimental" not in attr["name"]
+    ) and attr["name"] != "experiments":
         print("use experiment system for experiments")
         error = True
     if "description" not in attr:
@@ -176,7 +178,7 @@ attrs_in_packing_order = sorted(
     attrs, key=lambda a: SORT_ORDER_FOR_PACKING[a["type"]]
 )
 
-with open("test/core/util/fuzz_config_vars.proto", "w") as P:
+with open("test/core/test_util/fuzz_config_vars.proto", "w") as P:
     put_copyright(P)
 
     put_banner(
@@ -199,7 +201,7 @@ with open("test/core/util/fuzz_config_vars.proto", "w") as P:
         print(
             "  optional %s %s = %d;"
             % (
-                PROTO_TYPE[attr["type"]],
+                attr.get("fuzz_type", PROTO_TYPE[attr["type"]]),
                 attr["name"],
                 binascii.crc32(attr["name"].encode("ascii")) & 0x1FFFFFFF,
             ),
@@ -207,7 +209,7 @@ with open("test/core/util/fuzz_config_vars.proto", "w") as P:
         )
     print("};", file=P)
 
-with open("test/core/util/fuzz_config_vars.h", "w") as H:
+with open("test/core/test_util/fuzz_config_vars.h", "w") as H:
     put_copyright(H)
 
     put_banner(
@@ -219,12 +221,12 @@ with open("test/core/util/fuzz_config_vars.h", "w") as H:
         ],
     )
 
-    print("#ifndef GRPC_TEST_CORE_UTIL_FUZZ_CONFIG_VARS_H", file=H)
-    print("#define GRPC_TEST_CORE_UTIL_FUZZ_CONFIG_VARS_H", file=H)
+    print("#ifndef GRPC_TEST_CORE_TEST_UTIL_FUZZ_CONFIG_VARS_H", file=H)
+    print("#define GRPC_TEST_CORE_TEST_UTIL_FUZZ_CONFIG_VARS_H", file=H)
     print(file=H)
     print("#include <grpc/support/port_platform.h>", file=H)
     print(file=H)
-    print('#include "test/core/util/fuzz_config_vars.pb.h"', file=H)
+    print('#include "test/core/test_util/fuzz_config_vars.pb.h"', file=H)
     print('#include "src/core/lib/config/config_vars.h"', file=H)
     print(file=H)
     print("namespace grpc_core {", file=H)
@@ -243,9 +245,9 @@ with open("test/core/util/fuzz_config_vars.h", "w") as H:
     print(file=H)
     print("}  // namespace grpc_core", file=H)
     print(file=H)
-    print("#endif  // GRPC_TEST_CORE_UTIL_FUZZ_CONFIG_VARS_H", file=H)
+    print("#endif  // GRPC_TEST_CORE_TEST_UTIL_FUZZ_CONFIG_VARS_H", file=H)
 
-with open("test/core/util/fuzz_config_vars.cc", "w") as C:
+with open("test/core/test_util/fuzz_config_vars.cc", "w") as C:
     put_copyright(C)
 
     put_banner(
@@ -257,8 +259,8 @@ with open("test/core/util/fuzz_config_vars.cc", "w") as C:
         ],
     )
 
-    print('#include "test/core/util/fuzz_config_vars.h"', file=C)
-    print('#include "test/core/util/fuzz_config_vars_helpers.h"', file=C)
+    print('#include "test/core/test_util/fuzz_config_vars.h"', file=C)
+    print('#include "test/core/test_util/fuzz_config_vars_helpers.h"', file=C)
     print(file=C)
     print("namespace grpc_core {", file=C)
     print(file=C)
@@ -325,7 +327,7 @@ with open("src/core/lib/config/config_vars.h", "w") as H:
     print(file=H)
     print("namespace grpc_core {", file=H)
     print(file=H)
-    print("class ConfigVars {", file=H)
+    print("class GPR_DLL ConfigVars {", file=H)
     print(" public:", file=H)
     print("  struct Overrides {", file=H)
     for attr in attrs_in_packing_order:
