@@ -24,8 +24,8 @@ import sys
 
 import yaml
 
-with open("src/core/lib/debug/stats_data.yaml") as f:
-    attrs = yaml.safe_load(f.read(), Loader=yaml.Loader)
+with open("src/core/telemetry/stats_data.yaml") as f:
+    attrs = yaml.safe_load(f.read())
 
 REQUIRED_FIELDS = ["name", "doc"]
 
@@ -259,7 +259,7 @@ def snake_to_pascal(name):
     return "".join([x.capitalize() for x in name.split("_")])
 
 
-with open("src/core/lib/debug/stats_data.h", "w") as H:
+with open("src/core/telemetry/stats_data.h", "w") as H:
     # copy-paste copyright notice from this file
     with open(sys.argv[0]) as my_source:
         copyright = []
@@ -280,14 +280,14 @@ with open("src/core/lib/debug/stats_data.h", "w") as H:
         [H], ["Automatically generated by tools/codegen/core/gen_stats_data.py"]
     )
 
-    print("#ifndef GRPC_SRC_CORE_LIB_DEBUG_STATS_DATA_H", file=H)
-    print("#define GRPC_SRC_CORE_LIB_DEBUG_STATS_DATA_H", file=H)
+    print("#ifndef GRPC_SRC_CORE_TELEMETRY_STATS_DATA_H", file=H)
+    print("#define GRPC_SRC_CORE_TELEMETRY_STATS_DATA_H", file=H)
     print(file=H)
     print("#include <grpc/support/port_platform.h>", file=H)
     print("#include <atomic>", file=H)
     print("#include <memory>", file=H)
     print("#include <stdint.h>", file=H)
-    print('#include "src/core/lib/debug/histogram_view.h"', file=H)
+    print('#include "src/core/telemetry/histogram_view.h"', file=H)
     print('#include "absl/strings/string_view.h"', file=H)
     print('#include "src/core/lib/gprpp/per_cpu.h"', file=H)
     print(file=H)
@@ -302,6 +302,10 @@ with open("src/core/lib/debug/stats_data.h", "w") as H:
         print(" public:", file=H)
         print("  static int BucketFor(int value);", file=H)
         print("  const uint64_t* buckets() const { return buckets_; }", file=H)
+        print(
+            "  size_t bucket_count() const { return %d; }" % shape.buckets,
+            file=H,
+        )
         print(
             "  friend Histogram_%d_%d operator-(const Histogram_%d_%d& left,"
             " const Histogram_%d_%d& right);"
@@ -441,9 +445,9 @@ with open("src/core/lib/debug/stats_data.h", "w") as H:
     print("}", file=H)
 
     print(file=H)
-    print("#endif // GRPC_SRC_CORE_LIB_DEBUG_STATS_DATA_H", file=H)
+    print("#endif // GRPC_SRC_CORE_TELEMETRY_STATS_DATA_H", file=H)
 
-with open("src/core/lib/debug/stats_data.cc", "w") as C:
+with open("src/core/telemetry/stats_data.cc", "w") as C:
     # copy-paste copyright notice from this file
     with open(sys.argv[0]) as my_source:
         copyright = []
@@ -466,7 +470,7 @@ with open("src/core/lib/debug/stats_data.cc", "w") as C:
 
     print("#include <grpc/support/port_platform.h>", file=C)
     print(file=C)
-    print('#include "src/core/lib/debug/stats_data.h"', file=C)
+    print('#include "src/core/telemetry/stats_data.h"', file=C)
     print("#include <stdint.h>", file=C)
     print(file=C)
 
